import 'dart:convert';
import 'dart:io';

import 'package:flutter/foundation.dart';
import 'package:flutter/gestures.dart';
import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';
import 'package:flutter/services.dart';
import 'package:livelike_flutter_sdk/src/constants.dart';
import 'package:livelike_flutter_sdk/src/model/livelike_widget.dart';
import 'package:livelike_flutter_sdk/src/session.dart';

class WidgetView extends StatefulWidget {
  WidgetView(
      {Key? key,
      required this.session,
      this.widgetHeight = 350,
      this.widgetListener})
      : super(key: key);
  final Session session;
  final double widgetHeight;
  final void Function(LiveLikeWidget widget)? widgetListener;

  @override
  WidgetViewState createState() => WidgetViewState();
}

class WidgetViewState extends State<WidgetView> {
  final Map<String, dynamic> creationParams = <String, dynamic>{};
  bool _showWidget = true;
  double? widgetHeight;

  MethodChannel? _channel;

  @override
  Widget build(BuildContext context) {
    return SizedBox(
      height: _showWidget ? (widgetHeight ?? widget.widgetHeight) : 0,
      child: Platform.isAndroid
          ? PlatformViewLink(
              surfaceFactory: (context, controller) {
                return AndroidViewSurface(
                  controller: controller as AndroidViewController,
                  gestureRecognizers: const <
                      Factory<OneSequenceGestureRecognizer>>{},
                  hitTestBehavior: PlatformViewHitTestBehavior.opaque,
                );
              },
              onCreatePlatformView: (params) {
                return PlatformViewsService.initSurfaceAndroidView(
                  id: params.id,
                  viewType: widgetViewTypeKey,
                  layoutDirection: TextDirection.ltr,
                  creationParams: creationParams,
                  creationParamsCodec: const StandardMessageCodec(),
                )
                  ..addOnPlatformViewCreatedListener((id) async {
                    _channel = new MethodChannel('$widgetViewTypeKey$id');
                    await _channel?.invokeMethod(setSessionKey,
                        {programIDKey: widget.session.programId});
                    params.onPlatformViewCreated(id);
                    _streamListener();
                  })
                  ..create();
              },
              viewType: widgetViewTypeKey)
          : Platform.isIOS
              ? UiKitView(
                  viewType: widgetViewTypeKey,
                  layoutDirection: TextDirection.ltr,
                  creationParams: creationParams,
                  creationParamsCodec: const StandardMessageCodec(),
                  gestureRecognizers: const <
                      Factory<OneSequenceGestureRecognizer>>{},
                  hitTestBehavior: PlatformViewHitTestBehavior.opaque,
                  onPlatformViewCreated: (id) async {
                    _channel = new MethodChannel('$widgetViewTypeKey$id');
                    await _channel?.invokeMethod(setSessionKey,
                        {programIDKey: widget.session.programId});
                    _streamListener();
                  },
                )
              : SizedBox.shrink(),
    );
  }

  Future<void> displayWidget(LiveLikeWidget liveLikeWidget) async {
    await _channel?.invokeMethod(
        "displayWidget", jsonEncode(liveLikeWidget.toJson()));
  }

  Future<void> setWidgetState(WidgetState state) async {
    await _channel
        ?.invokeMethod("setState", {"state": state.toString().split(".")[1]});
  }

  Future<void> enableDefaultWidgetTransition(bool enable) async {
    await _channel
        ?.invokeMethod("enableDefaultWidgetTransition", {"enable": enable});
  }

  void _streamListener() async {
    await Future.delayed(const Duration(seconds: 2));
    if (mounted)
      setState(() {
        _showWidget = false;
      });
    widget.session.widgetStream.listen((event) async {
      if (mounted) {
        widget.widgetListener?.call(event);
        print(
            "Show: ${event.show} , Height:${event.widget_height}> ${MediaQuery.of(context).devicePixelRatio}");
        setState(() {
          _showWidget = (event.show ?? false);
          if (event.widget_height != null)
            widgetHeight =
                (event.widget_height ?? 0) + (Platform.isIOS ? 30 : 0);
          else
            widgetHeight = null;
        });
      }
    });
  }
}
